
% read an image (gray-scale)
y = imread('low_contrast.jpg');

% setup function for contrast manipulation
c = 1.7;
adjustContrast = @(x) c*x(2,2)+(1.0-c)*(mean(x(:)-x(2,2)/9.0));

% distribute image by columns
yd = distributed(y);

% now work in parallel
spmd
    % get local part
    yl = getLocalPart(yd);

    % compute neighbours
    next     = mod(labindex,numlabs) + 1   % worker to the right
    previous = mod(labindex-2,numlabs) +1  % worker to the left
   
    % exchange boundaries to left
    column = labSendReceive(previous, next, yl(:,1));
    if (labindex() < numlabs())
        yl = [yl, column];
    end
    % exchange boundaries to right
    column = labSendReceive(next, previous, yl(:,end-1));
    if (labindex() > 1)
        yl = [column, yl];
    end
    
    % apply filter
    yl = nlfilter(yl, [3,3], adjustContrast);

    % discard extra columns for other workers
    if ( labindex() < numlabs() )  % unless I am last worker
        yl = yl(:,1:end-1);        % remove extra column on right
    end
    if ( labindex() > 1 )          % unless I am first worker
        yl = yl(:,2:end);          % remove extra column on left
    end
    
end

% combine local images
z = [ yl{:}];

% save image side-by side
imwrite(cat(1,y,z), 'contrast_spmd_v2.jpg');

